@@ -1,18 +1,18 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhtxthtml.pl 2.9 99/08/14 19:38:33
+##	@(#) mhtxthtml.pl 2.13 00/10/28 10:55:44
 ##  Author:
 ##      Earl Hood       mhonarc@pobox.com
 ##  Description:
 ##	Library defines routine to filter text/html body parts
 ##	for MHonArc.
 ##	Filter routine can be registered with the following:
-##		<MIMEFILTERS>
-##		text/html:m2h_text_html'filter:mhtxthtml.pl
-##		</MIMEFILTERS>
+##	    <MIMEFILTERS>
+##	    text/html:m2h_text_html'filter:mhtxthtml.pl
+##	    </MIMEFILTERS>
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-1999	Earl Hood, mhonarc@pobox.com
+##    Copyright (C) 1995-2000	Earl Hood, mhonarc@pobox.com
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -38,6 +38,9 @@ my $Url	= '(\w+://|\w+:)';
 my $SAttr = q/(?:onload|onunload|onclick|ondblclick|/.
 	    q/onmouse(?:down|up|over|move|out)|/.
 	    q/onkey(?:press|down|up))/;
+# Script/questionable related elements
+my $SElem = q/(?:applet|base|embed|form|ilayer|input|layer|link|meta|object|/.
+	    q/option|param|select|textarea)/;
 
 ##---------------------------------------------------------------------------
 ##	The filter must modify HTML content parts for merging into the
@@ -51,6 +54,8 @@ my $SAttr = q/(?:onload|onunload|onclick|ondblclick|/.
 ##			to scripting.  The default is to delete any
 ##			scripting markup for security reasons.
 ##
+##	nofont  	Remove <FONT> tags.
+##
 sub filter {
     local($header, *fields, *data, $isdecode, $args) = @_;
     local(@files) = ();	# !!!Used by resolve_cid!!!
@@ -58,8 +63,13 @@ sub filter {
     my $title	 = '';
     my $noscript = 1;
        $noscript = 0  if $args =~ /\ballowscript\b/i;
+    my $nofont	 = $args =~ /\bnofont\b/i;
     my $tmp;
 
+    ## Remove comment declarations: may screw-up mhonarc processing
+    ## and avoids someone sneaking in SSIs.
+    $data =~ s/<!(?:--(?:[^-]|-[^-])*--\s*)+>//go;
+
     ## Get/remove title
     if ($data =~ s|<title\s*>([^<]*)</title\s*>||io) {
         $title = "<ADDRESS>Title: <STRONG>$1</STRONG></ADDRESS>\n";
@@ -84,17 +94,24 @@ sub filter {
 
     ## Strip out certain elements/tags to support proper inclusion
     $data =~ s|<!doctype\s[^>]*>||io;
-    $data =~ s|</?html[^>]*>||gio;
+    $data =~ s|</?html\b[^>]*>||gio;
+    $data =~ s|</?x-html\b[^>]*>||gio;
     $data =~ s|<head\s*>[\s\S]*</head\s*>||io;
 
+    ## Strip out <font> tags if requested
+    if ($nofont) {
+	$data =~ s|<style[^>]*>.*?</style\s*>||gios;
+	$data =~ s|</?font\b[^>]*>||gio;
+    }
+
     ## Strip out scripting markup if requested
     if ($noscript) {
 	$data =~ s|<script[^>]*>.*?</script\s*>||gios;
-	$data =~ s|\b$SAttr\b\s*=\s*"[^"]*"||gio;
-	$data =~ s|\b$SAttr\b\s*=\s*'[^']*'||gio;
+	$data =~ s|<style[^>]*>.*?</style\s*>||gios  unless $nofont;
+	$data =~ s|\b$SAttr\b\s*=\s*"[^"]*"||gio; #"
+	$data =~ s|\b$SAttr\b\s*=\s*'[^']*'||gio; #'
 	$data =~ s|\b$SAttr\b\s*=\s*[^\s>]+||gio;
-	$data =~ s|</?applet[^>]*>||gio;
-	$data =~ s|</?param[^>]*>||gio;
+	$data =~ s|</?$SElem[^>]*>||gio;
     }
 
     ## Check for body attributes
@@ -107,13 +124,13 @@ sub filter {
 	    ## set visual properties.  We use a mixture of old attributes
 	    ## and CSS to set properties since browsers may not support
 	    ## all of the CSS settings via the STYLE attribute.
-	    my $tpre = '<TABLE><TR><TD ';
+	    my $tpre = '<table width="100%"><tr><td ';
 	    my $tsuf = "";
-	    $tpre .= qq|BACKGROUND="$attr{'background'}" |
+	    $tpre .= qq|background="$attr{'background'}" |
 		     if $attr{'background'};
-	    $tpre .= qq|BGCOLOR="$attr{'bgcolor'}" |
+	    $tpre .= qq|bgcolor="$attr{'bgcolor'}" |
 		     if $attr{'bgcolor'};
-	    $tpre .= qq|STYLE="|;
+	    $tpre .= qq|style="|;
 	    $tpre .= qq|background-color: $attr{'bgcolor'}; |
 		     if $attr{'bgcolor'};
 	    if ($attr{'background'}) {
@@ -126,18 +143,18 @@ sub filter {
 	    }
 	    $tpre .= qq|color: $attr{'text'}; |
 		     if $attr{'text'};
-	    $tpre .= qq|A:link { color: $attr{'link'} } |
+	    $tpre .= qq|a:link { color: $attr{'link'} } |
 		     if $attr{'link'};
-	    $tpre .= qq|A:active { color: $attr{'alink'} } |
+	    $tpre .= qq|a:active { color: $attr{'alink'} } |
 		     if $attr{'alink'};
-	    $tpre .= qq|A:visited { color: $attr{'vlink'} } |
+	    $tpre .= qq|a:visited { color: $attr{'vlink'} } |
 		     if $attr{'vlink'};
 	    $tpre .= '">';
 	    if ($attr{'text'}) {
-		$tpre .= qq|<FONT COLOR="$attr{'text'}">|;
-		$tsuf .= '</FONT>';
+		$tpre .= qq|<font color="$attr{'text'}">|;
+		$tsuf .= '</font>';
 	    }
-	    $tsuf .= '</TD></TR></TABLE>';
+	    $tsuf .= '</td></tr></table>';
 	    $data = $tpre . $data . $tsuf;
 	}
     }
@@ -150,9 +167,9 @@ sub filter {
     }
 
     ## Check for CID URLs (multipart/related HTML)
-    $data =~ s/((?:href|src|background)\s*=\s*['"])cid:([^'"]+)(['"])/
+    $data =~ s/((?:href|src|background)\s*=\s*['"])([^'"]+)(['"])/
 	       join("", $1, &resolve_cid($2), $3)/geix;
-    $data =~ s/((?:href|src|background)\s*=\s*)cid:([^\s>]+)/
+    $data =~ s/((?:href|src|background)\s*=\s*)([^'">][^\s>]+)/
 	       join("", $1, '"', &resolve_cid($2), '"')/geix;
 
     ($title.$data, @files);
@@ -162,6 +179,8 @@ sub filter {
 
 sub addbase {
     my($b, $u) = @_;
+    return $u  if !defined($b) || $b !~ /\S/;
+
     my($ret);
     $u =~ s/^\s+//;
     if ($u =~ m%^$Url%o || $u =~ m/^#/) {
@@ -175,7 +194,8 @@ sub addbase {
 	    $b =~ s/\/$//;
 	    my @a = split(/\//, $b);
 	    my $cnt = 0;
-	    while ($u =~ s|^(\.{1,2})/||) { ++$cnt  if length($1) == 2; }
+	    while ( $cnt <= scalar(@a) &&
+		    $u =~ s|^(\.{1,2})/|| ) { ++$cnt  if length($1) == 2; }
 	    splice(@a, -$cnt)  if $cnt > 0;
 	    $b = join('/', @a, "");
 
@@ -193,7 +213,7 @@ sub addbase {
 sub resolve_cid {
     my $cid = shift;
     my $href = $readmail::Cid{$cid};
-    if (!defined($href)) { return "cid:$cid"; }
+    if (!defined($href)) { return $cid; }
 
     require 'mhmimetypes.pl';
     my $filename;
@@ -208,7 +228,7 @@ sub resolve_cid {
 			    $href->{'fields'}{'content-type'},
 			    \$href->{'body'});
     }
-    $href->{'filtered'} = 1;
+    $href->{'filtered'} = 1; # mark part filtered for readmail.pl
     push(@files, $filename); # @files defined in filter
     $filename;
 }
